// ThingWorx
// 
// получение данных с датчиков, подключенных к Arduino
// - температура воздуха LM35
// - влажность воздуха DHT11 (DHT22)
// - температура почвы DS18B20 герматичный
// - влажность почвы YL-38 soilMoisture
// - освещенность - фоторезистор
//
//  отправка в ThingWorx
//  для отображения в mushup Agro2016_Example

// *********  константы ********
#include "defines.h"
// *********  подключение библиотек  *********

// ds18b20
#include <OneWire.h>
// dht
#include "DHT.h"
// Ethernet
#include <SPI.h>
#include <Ethernet.h>

DHT dht(PIN_DHT, DHTTYPE);  // тип датчика DHT11,DHT22
OneWire  ds(PIN_DS18B20_1);  // подключение onewire датчиков температуры
// для DS18B20
byte addr[2][8]={
                {0x28, 0x90, 0x57, 0x65, 0x4, 0x0, 0x0, 0xE4},
                {0x28, 0xE3, 0x4, 0x67, 0x5, 0x0, 0x0, 0x52}
                };
                
byte arduino_mac[] = { 0xFF, 0xFE, 0xFB, 0xFA, 0xF9, 0xF8 };
EthernetClient client;
// если адрес статический
// (установить #define DHCP 1)
int mip[4]={192,168,0,121};
int mmask[4]={255,255,255,0};
int mgateway[4]={192,168,0,28};
int mdns[4]={192,168,1,1};

// ThingWorx сервер
//char server[] = "thingworx-academic-staff.ptcmscloud.com";
char server[] = "tvsn.cloud.thingworx.com";
// ThingWorx App key 
//char appKey[] = "9bcf4e6c-ce7c-48c6-9b5e-59634f58066e";
char appKey[] = "87e241cc-4b5a-4080-ab6e-8aba62047c47";
// ThingWorx Thing name 
char thingName[] = "ThingGreenhouse_i00019_01";
// Интервал отправки данных 
unsigned long timeBetweenRefresh = 30000;
// Интервал получения статуса checkbox для полива 
unsigned long timeBetweenRefresh2 = 5000;
// ThingWorx service 
char serviceName[] = "ServiceGreenhouse_01";
// Initialize Properties Names and Values Arrays
//How many values you will be pushing to ThingWorx
#define propertyCount 7
char* propertyNames[] = {"ValLat","ValLon",
                         "ValAirTemperature", "ValAirHumidity", 
                         "ValSoilTemperature", "ValSoilMoisture",
                         "ValLight",
                         };
double propertyValues[propertyCount];
int digitValues[propertyCount]={4,4,2,2,2,2,2};

// last time you connected to the server, in milliseconds
unsigned long lastConnectionTime = 0;
unsigned long lastConnectionTime2 = 0;
// state of the connection last time through the main loop
boolean lastConnected = false;
boolean lastConnected2 = false;


float fvalue;     // переменная

void setup() {
  Serial.begin(9600);
  // инициализация dht11
  dht.begin();
  // установка сетевого соединения
  ethernet_begin();
  print_ip();
  pinMode(PIN_PUMP,OUTPUT);
  digitalWrite(PIN_PUMP,LOW);  // выключить помпу
 }



void loop() 
  {
  // опрос состояния checkbox - кнопки полива 
  if (millis() - lastConnectionTime2 > timeBetweenRefresh2) 
    {
      
    int water = getValue("Watering");
    if (water > 998) 
       {
      Serial.println("GET parse error");
       } 
    else 
      {
      Serial.print("Received WATER= ");
      Serial.println(water);
      digitalWrite(PIN_PUMP,1-water);
      }
    /*delay(500);
    int fan = getValue("Fan");
    if (fan > 998) {
      Serial.println("GET parse error");
    } else {
      Serial.print("Received FAN= ");
      Serial.println(fan);
    }
    delay(500);*/
    }
  // ***** получение показаний с датчиков
  if (millis() - lastConnectionTime > timeBetweenRefresh) 
    {
    // ds18b20 - 1
    propertyValues[2]=get_temp(0);
    // dht11
    propertyValues[3]=get_dht();
    // lm35
    propertyValues[4]=get_lm35();
    // soilMoisture
    propertyValues[5]=get_soilMoisture();
    // фоторезистор
    propertyValues[6]=get_light();
    
    propertyValues[0]=44.1365;
    propertyValues[1]=43.0278;

    updateValues(propertyValues, client, server, appKey, thingName, serviceName, propertyNames);
    }
  
  }

 

