#include <SPI.h>
#include <Ethernet.h>

// уникальный MAC-адрес платы Arduino
byte mac[] = { 0xD4, 0x28, 0xB2, 0xFF, 0xA0, 0xA1 }; 

// IP адрес, назначаемый Ethernet shield:
byte ip[] = { 192, 168, 0, 119 };    
// IP адрес, dns сервера:
byte sdns[] = { 192, 168, 1, 1 };    
// адрес шлюза:
byte gateway[] = { 192, 168, 0, 28 };
// маска:
byte subnet[] = { 255, 255, 255, 0 };
 
// ThingSpeak Settings
char thingSpeakAddress[] = "api.thingspeak.com";
String writeAPIKey = "P8VDU06MDIMC3MAM";
// Интервал отправки данных на сервер – 16 сек
const int updateThingSpeakInterval = 16 * 1000;      

// служебные переменные
long lastConnectionTime = 0; 
boolean lastConnected = false;
int failedCounter = 0;

// Инициализация Arduino Ethernet client
EthernetClient client;

void setup()
  {
  // Запуск последовательного порта
  Serial.begin(9600);
  // Запуск Ethernet на Arduino
  startEthernet();
  }

void loop()
  {
  // Чтение данных из A0 и A1
  String analogValue0 = String(analogRead(A0), DEC);
  double temp335=analogRead(A1)*5.0/1024*100-273.15;
  String analogValue1 = String(temp335);
  // печать запроса в последовательный порт
  if (client.available())
    {
    char c = client.read();
    Serial.print(c);
    }

  // Разъединение с ThingSpeak
  if (!client.connected() && lastConnected)
    {
    Serial.println("...disconnected");
    Serial.println();
    client.stop();
    }
  
  // отправка данных в Канал ThingSpeak
  if(!client.connected() && (millis() - lastConnectionTime > updateThingSpeakInterval))
    {
    updateThingSpeak("field1="+analogValue0+"&field2="+analogValue1);
    Serial.println(analogValue0);
    Serial.println(analogValue1);
    }
  // При кол-ве неуспешных попыток >10 – перезапуск интернет-соединения
  if (failedCounter > 10 ) {startEthernet();}
  lastConnected = client.connected();
}
// отправка данных в Канал Thingspeak
void updateThingSpeak(String tsData)
  {
  if (client.connect(thingSpeakAddress, 80))
    {         
    client.print("POST /update HTTP/1.1\n");
    client.print("Host: api.thingspeak.com\n");
    client.print("Connection: close\n");
    client.print("X-THINGSPEAKAPIKEY: "+writeAPIKey+"\n");
    client.print("Content-Type: application/x-www-form-urlencoded\n");
    client.print("Content-Length: ");
    client.print(tsData.length());
    client.print("\n\n");

    client.print(tsData);
    lastConnectionTime = millis();
    if (client.connected())
      {
      Serial.println("Connecting to ThingSpeak...");
      Serial.println();
      
      failedCounter = 0;
      }
    else
      {
      failedCounter++;  
      Serial.println("Connection to ThingSpeak failed ("+String(failedCounter, DEC)+")");   
      Serial.println();
      }
    }
  else
    {
    // увеличение счетчика неуспешных попыток отправки данных
    failedCounter++;
    Serial.println("Connection to ThingSpeak Failed ("+String(failedCounter, DEC)+")");   
    Serial.println();
    lastConnectionTime = millis(); 
    }
  }
// Перезапуск интернет-соединения
void startEthernet()
  {
  client.stop();
  Serial.println("Connecting Arduino to network...");
  Serial.println();  
  delay(1000);
  Ethernet.begin(mac,ip,sdns,gateway,subnet); 
  delay(1000);
    Serial.print("My IP address: ");
  for (byte thisByte = 0; thisByte < 4; thisByte++)
   {
    // печать ip-адреса
    Serial.print(Ethernet.localIP()[thisByte], DEC);
    Serial.print(".");
    }
  Serial.println(); 
  delay(1000);
}

