// константы
#define DSD18B20PIN 5      // пин подключения контакта DATA (D1, GPIO5)
#define DHTPIN 4           // пин подключения контакта DATA
#define DHTTYPE DHT11      // датчик DHT 11

// подключение библиотеки
// http://www.pjrc.com/teensy/td_libs_OneWire.html
#include <OneWire.h>
// создание экземпляра OneWire
OneWire  ds(DSD18B20PIN); 
// подключение библиотеки DHT
#include "DHT.h"
// создание экземпляра объекта DHT
DHT dht(DHTPIN,DHTTYPE);

// подключение библиотеки WiFi
#include <ESP8266WiFi.h>
// Wi-Fi Settings
// wireless network name (SSID)
const char* ssid = "MacBook-Pro-Victor";
// Wi-Fi network password
const char* password = "19101966"; 

WiFiClient client;

// ThingSpeak Settings
const int channelID = 319811;
String writeAPIKey = "CB09FMVF9ERI9CCT"; // write API key for your ThingSpeak Channel
const char* server = "api.thingspeak.com";
const int postingInterval = 20 * 1000; // post data every 20 seconds

void setup(void) {
  // запуск последовательного порта
  Serial.begin(115200);
  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
  }
  Serial.println("");
  Serial.println("WiFi connected");  
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
}

void loop() {

  if (client.connect(server, 80)) {
    // Construct API request body
    String body = "field2=";
    int humidity = get_data_humidity();    
    body += String(humidity);
    float temp = get_data_ds18b20();  
    if(temp<999) {  
       body += "&field1=";
       body += String(temp);
    }
    client.print("POST /update HTTP/1.1\n");
    client.print("Host: api.thingspeak.com\n");
    client.print("Connection: close\n");
    client.print("X-THINGSPEAKAPIKEY: " + writeAPIKey + "\n");
    client.print("Content-Type: application/x-www-form-urlencoded\n");
    client.print("Content-Length: ");
    client.print(body.length());
    client.print("\n\n");
    client.print(body);
    client.print("\n\n");

  }
  client.stop();
      


  // wait and then post again
  delay(postingInterval);
}

float get_data_ds18b20(void) {
  byte i;
  byte present = 0;
  byte type_s;
  byte data[12];
  byte addr[8];
  float fTemp;
  
  if ( !ds.search(addr)) {
    Serial.println("No more addresses.");
    Serial.println();
    ds.reset_search();
    delay(250);
    return 999;
  }
  
  Serial.print("ROM =");
  for( i = 0; i < 8; i++) {
    Serial.write(' ');
    Serial.print(addr[i], HEX);
  }

  if (OneWire::crc8(addr, 7) != addr[7]) {
      Serial.println("CRC is not valid!");
      return 999;
  }
  Serial.println();
 
  // the first ROM byte indicates which chip
  switch (addr[0]) {
    case 0x10:
      Serial.println("  Chip = DS18S20");  // or old DS1820
      type_s = 1;
      break;
    case 0x28:
      Serial.println("  Chip = DS18B20");
      type_s = 0;
      break;
    case 0x22:
      Serial.println("  Chip = DS1822");
      type_s = 0;
      break;
    default:
      Serial.println("Device is not a DS18x20 family device.");
      return 999;
  } 

  ds.reset();
  ds.select(addr);
  // запустить конвертацию температуры датчиком
  ds.write(0x44, 1);          
  delay(1000);     // ждем 750 мс
  present = ds.reset();
  ds.select(addr);    
  ds.write(0xBE);       
  // считываем ОЗУ датчика
  for ( i = 0; i < 9; i++) {           
    data[i] = ds.read();
    Serial.print(data[i], HEX);
    Serial.print(" ");
  }
  // перевод полученных данных в значение температуры
  int16_t raw = (data[1] << 8) | data[0];
  if (type_s) {
    raw = raw << 3; 
    if (data[7] == 0x10) {
      raw = (raw & 0xFFF0) + 12 - data[6];
    }
  } else {
    byte cfg = (data[4] & 0x60);
    if (cfg == 0x00) raw = raw & ~7;  
    else if (cfg == 0x20) raw = raw & ~3; 
    else if (cfg == 0x40) raw = raw & ~1; 
  }
  fTemp = (float)raw / 16.0;

  return fTemp;
}

int get_data_humidity() {
   int h = dht.readHumidity();
   return h;
}
